import json
import logging
import sys

import click
import hvac
from psycopg2.extras import NumericRange
from stumpf.app import create_app
from stumpf.diva_models import Activity, ActivityTrack, Detection, Track
from stumpf.models import db, Video


@click.group(name='diva', short_help='The DIVA related commands.')
@click.option('--production', is_flag=True, default=False,
              help='Use the production Stumpf database.')
@click.option('--vault-token', type=click.UUID, help='The token to use to authenticate to Vault.')
@click.pass_obj
def diva(ctx, production, vault_token):
    ctx['production'] = production
    ctx['vault_token'] = str(vault_token)


@diva.command(name='import-grouper',
              help='Import activities, tracks, and detections from Grouper JSON.')
@click.argument('video-id', type=click.UUID)
@click.argument('infiles', nargs=-1, type=click.File(), metavar='FILE')
@click.pass_obj
def import_grouper(ctx, video_id, infiles):
    logging.getLogger('flask.app').setLevel(logging.ERROR)
    app_config = {
        'SQLALCHEMY_ECHO': False,
    }

    if ctx['production']:
        client = hvac.Client(url='https://vault.kitware.com', token=ctx['vault_token'])
        db_creds = client.read('secret/data/projects/diva-mturk/stumpf/rds')['data']['data']
        app_config['SQLALCHEMY_DATABASE_URI'] = \
            f'postgresql://{db_creds["username"]}:{db_creds["password"]}@{db_creds["host"]}/{db_creds["dbname"]}'  # noqa

    app = create_app(app_config)

    with app.app_context():
        if not Video.query.get(video_id):
            click.echo(f'Video with ID {video_id} does not exist.', err=True)
            sys.exit(1)

        for infile in infiles:
            data = json.load(infile)

            # build activities
            frame_start = data['detections'][0]['ts0']
            frame_end = data['detections'][1]['ts0']

            activity = Activity(video_id=video_id, type_=data['type'],
                                frame_range=NumericRange(frame_start, frame_end, bounds='[]'))
            db.session.add(activity)
            db.session.commit()

            # build tracks/detections
            for track in data['tracks']:
                t = Track(video_id=video_id, type_=track['type'])
                at = ActivityTrack(activity=activity, track=t)
                db.session.add_all([t, at])

                for detection in track['detections']:
                    det = Detection(video_id=video_id, track=t, source=detection['src'],
                                    frame=detection['ts0'], occlusion='none',
                                    bounds=map(int, detection['g0'].split()))
                    db.session.add(det)

                db.session.commit()
