import click
import json
import time
import sys

DELAY = 2  # delay between posting tasks for AWS rate limiting


def valid_json(value):
    # https://github.com/pallets/click/issues/950
    try:
        args = json.loads(value)
        return args
    except Exception:
        raise click.BadParameter('args should be valid json')


@click.group(name='task', short_help='The task resource.')
@click.pass_obj
def task(ctx):
    pass


@task.command(name='generate', help='Generate manifest file for task(s)')
@click.argument('type', type=click.Choice(['HIT', 'KWIT']))
@click.argument('video_id')
@click.argument('app_id')
@click.argument('args', default='{}')
@click.option('-o','--output', type=click.File(mode='a'), default=sys.stdout)
@click.pass_obj
def generate(ctx, type, video_id, app_id, args, output):
    args = valid_json(args)
    url = f'task/{video_id}/{app_id}/{type.lower()}'
    output.write(json.dumps({
        'url': url,
        'args': args,
    }) + '\n')


@task.command(name='submit', help='Submit generated task(s)')
@click.argument('input', type=click.File(mode='r'), nargs=-1)
@click.pass_obj
def submit(ctx, input):

    def request(source, finished, failed):
        for line in source:
            task = json.loads(line.strip('\n'))
            r = ctx['session'].post(task['url'], json=task['args'])
            if not r.ok: # Not reliably json
                failed.write(r.text + '\n')
            else:
                finished.write(json.dumps(r.json()) + '\n')
            # AWS rate limiting
            time.sleep(2)

    if len(input) == 0:
        request(sys.stdin, sys.stdout, sys.stderr)
    else:
        for source in input:
            with open(f'{source.name}_finished', 'w') as finished, \
                    open(f'{source.name}_failed', 'w') as failed:
                request(source, finished, failed)


@task.command(name='list', help='List tasks')
@click.pass_obj
def list_(ctx):
    r = ctx['session'].get('task')

    if not r.ok:
        click.echo(json.dumps(r.json()), err=True)
    else:
        click.echo(json.dumps(r.json(), indent=4))
